#include <string>
#include <sstream>
#include "windows.h"
#include "HeXHubSDK.h"
#include "sysinfo.h"

HANDLE CPUSecondUpdateTimer;
CRITICAL_SECTION Lock;
Sysinfo sys;
long trigId, hlpId;
FILETIME lastCPUSecond;
float PercentCPU;

void HEXHUBAPI onSystemInfoCmd(DWORD dwCommandId, DWORD userId, LPVOID rights, LPVOID commandParameters, BOOL isPM);
VOID CALLBACK UpdateCPUSeconds(LPVOID lpParam, BOOL b);

const char * HEXHUBAPI InitPlugin(lpfnGetFunction HexGetFunction) {
	//HexHubFunctions functions;
	HexHub Hex;
	
	if (Hex.Initialize(HexGetFunction) == false) return 0;

	if ((trigId = HexHub::Core::RegisterCommand("systeminfo", onSystemInfoCmd)) == -1) return 0;
	
	if ((hlpId = HexHub::Core::RegisterHelpString("EN", "!systeminfo\t\t\t-\tShows Core System Information", 0)) == -1) {
		HexHub::Core::UnregisterCommand(trigId);
		return 0;
	}

	CreateTimerQueueTimer(&CPUSecondUpdateTimer, NULL, (WAITORTIMERCALLBACK)UpdateCPUSeconds, NULL, 1000, 1000, NULL);

	InitializeCriticalSection(&Lock);

	return "System Information Plugin 0.4 by The Architect";
}

void HEXHUBAPI UnloadPlugin(void) {
	DeleteCriticalSection(&Lock);
	DeleteTimerQueueTimer(NULL, CPUSecondUpdateTimer, NULL);
}

const char * HEXHUBAPI GetAboutText(void) {
	return "Mikhail Polenin (The Architect) 2007-2008";
}

void HEXHUBAPI onSystemInfoCmd(DWORD dwCommandId,DWORD userId,LPVOID rights,LPVOID commandParameters,BOOL isPM)
{
	if(dwCommandId != trigId) return;
	
	std::string inf;

	EnterCriticalSection(&Lock);
	inf = sys.GetSysinfo();
	LeaveCriticalSection(&Lock);

	HexHub::Data::SendTo(userId, inf.c_str(), (int)inf.length());
}

VOID CALLBACK UpdateCPUSeconds(LPVOID lpParam, BOOL b)
{
	DWORD dwProcId = GetCurrentProcessId();
	HANDLE hProcess = OpenProcess(PROCESS_QUERY_INFORMATION, NULL, dwProcId);
	FILETIME ftCreation;
	FILETIME ftExit;
	FILETIME ftKernel;
	FILETIME ftUser;
	GetProcessTimes(hProcess, &ftCreation, &ftExit, &ftKernel, &ftUser);

	FILETIME temp;
	temp.dwLowDateTime = (ftKernel.dwLowDateTime + ftUser.dwLowDateTime) - lastCPUSecond.dwLowDateTime;
	lastCPUSecond.dwLowDateTime = ftKernel.dwLowDateTime + ftUser.dwLowDateTime;
	SYSTEM_INFO si;
	GetSystemInfo(&si);
	PercentCPU = sys.GetPercentage(temp.dwLowDateTime, 10000000) / si.dwNumberOfProcessors;
}